
package use_trafficintersection;

import java.util.ArrayList;
import java.util.Random;
import use_trafficintersection.algorithm.Car;
import use_trafficintersection.algorithm.Main;
import use_trafficintersection.algorithm.Lights;

/**
 *
 * @author Sjoerd Westendorp
 */
public class AutoTester {
    public static boolean useAutoTester = false;
    
    public static long TEST_CARCOUNT = 2*Main.TIME_SPENT_LIST_MAX_SIZE;
    
    // Other objects
    Main main;
    Renderer renderer;
    Lights lights;
    
    // Algo Tests
    public int algo_num = -1;
    public String algo = "";
    public static final String ALGOS_NORMAL[] = new String[]{
	"cost_auto_pl",
	"vac"
    };
    public static final String ALGOS_DEBUG[] = new String[]{
	"vac",
	"cost_auto_pl",
	"cost_auto",
	"cost",
	"fixed"
    };
    public static final String ALGOS[] = ALGOS_NORMAL;
    public long seed = 0;
    
    // Output
    public static ArrayList<String> output = new ArrayList<>();
    
    // -------------------------------
    
    public AutoTester(Main m, Renderer r,Lights l){
	main = m;
	renderer = r;
	lights = l;
    }
    
    // -------------------------------
    
    public void test(){
	testAlgo();
    }
    
    public void testThis(String algorithm){
	algo = algorithm;
	testAlgoStep();
    }
    
    public void testAlgo(){
	if( algo_num < 0){
	    // Random seed selector
	    Random rand = new Random();
	    seed = rand.nextLong();
	    // No algo set, lets test the first one
	    String str = "Cars: "+((int)renderer.renderWorld.sliderCars.state)+"%";
	    str += " | Autos: "+((int)renderer.renderWorld.sliderAutos.state)+"%";
	    str += " | Loss: "+((int)renderer.renderWorld.sliderPacketLoss.state)+"%";
	    str += " | Platoon: "+Car.usePlatooning;
	    log(str);
	    str = "Bias";
	    str += " | U: "+((int)renderer.renderWorld.sliderBiasU.state)+"%";
	    str += " | L: "+((int)renderer.renderWorld.sliderBiasL.state)+"%";
	    str += " | D: "+((int)renderer.renderWorld.sliderBiasD.state)+"%";
	    str += " | R: "+((int)renderer.renderWorld.sliderBiasR.state)+"%";
	    log(str);
	    str = "Seed: " + seed;
	    log(str);
	    algo_num = 0;
	    testSetupAlgo();
	    
	}
	if( algo_num >= ALGOS.length){
	    useAutoTester = false;
	    log("DONE"); log("");
	    algo_num = -1;
	    renderer.renderWorld.sliderSimSpeed.state = renderer.renderWorld.sliderSimSpeed.min;
	    lights.reset();
	    return; // We are done
	}
	
	// Check wether we are done with this algo
	if(main.carCount >= TEST_CARCOUNT){
	    String s = "["+algo.toUpperCase()+"] ";
	    s += " Delay: "+main.timeSpent;
	    s += " | Accel: "+Math.round(main.accel*100.0)/100.0;
	    s += " | Ped: "+main.timeSpentPed;
	    log(s);
	    
	    // Start next one
	    algo_num++;
	    testSetupAlgo();
	}
	
	// Try algorithm
	testAlgoStep();
    }
    
    private void testAlgoStep(){
	if( algo.equals("fixed") )
	    lights.algoFixed();
	else if( algo.equals("cost") )
	    lights.algoCost(false,false);
	else if( algo.equals("cost_auto") )
	    lights.algoCost(true,false);
	else if( algo.equals("cost_auto_pl") )
	    lights.algoCost(true,true);
	else if( algo.equals("vac") )
	    lights.algoVac();
	else{
	    System.out.print("[ERROR] lights.testAlgoStep -> Incorrect Algo num: "+algo);
	    System.out.println(", max:"+(ALGOS.length)+".");
	}
    }
    
    private void testSetupAlgo(){
	if( algo_num >= ALGOS.length){
	    return; // We are done
	}
	algo = ALGOS[algo_num];
	main.reset();
	renderer.renderWorld.sliderSimSpeed.state = renderer.renderWorld.sliderSimSpeed.max;
	// Random seed selector
	main.rand_uniform.setSeed(seed);
	main.rand_packetloss.setSeed(seed+1);
    }
    
    private void log(String s){
	output.add(s);
	System.out.println(s);
    }
}
