
package use_trafficintersection.algorithm;

//import use_trafficintersection.Renderer;

import java.awt.Color;
import use_trafficintersection.Renderer;

/**
 * @author Sjoerd Westendorp
 */

/*
This class defines the pedestrians
and their movement
*/

public class Ped {
    boolean isDone = false;
    long loopSpawned, loopExpected;
    
    // Other object
    Lights lights;
    
    // Constants
    public static final int STATE_PRE = 1;
    public static final int STATE_WAIT = 2;
    public static final int STATE_POST = 3;
    
    // Local constants
    public final float speed = 0.4f + 0.3f*Lights.main.rand_uniform.nextFloat();
    
    // Properties
    public int dirStart; // Direction it starts in (UP,DOWN,LEFT,RIGHT)
    public int dirEnd; // Direction it want to go in (UP,DOWN,LEFT,RIGHT)
    public int dirRel; // Relative direction it want to go in (GO_LEFT,GO_RIGHT,GO_FORWARD)
    public int ligth; // Light the ped has to wait on
    public int r;
    public Color c;
    
    // Movement
    public int state;
    public float rx, ry;
    public float x,y; // Coordinates
    public int x(){ return (int) (x+rx); }
    public int y(){ return (int) (y+ry); }
    
    // -----------------------------
    //		SETUP
    // -----------------------------
    
    public Ped(Lights l,int tr, int dir_start, int dir_rel, long loop){
        lights = l;
	r = tr;
	c = new Color(200,0,100);
	setDirections(dir_start,dir_rel);
	setStartingPos(dir_start);
	rx = -3.0f + 6.0f* (float) (Math.random());
	ry = -3.0f + 6.0f* (float) (Math.random());
	state = STATE_PRE;
        loopSpawned = loop;
        loopExpected = loop+(long)((Renderer.WORLD_SIZE/2 + Renderer.START_OFFSET)/speed) - 8;
    }
    
    private void setDirections(int dir_start, int direction){
	dirStart = dir_start; 
	dirRel = direction; 
	/*
	It is posseble for dir end to be either
	- A direction (UP,DOWN,LEFT,RIGHT)
	- Or a relative direction (GO_LEFT,GO_RIGHT,GO_FORWARD)
	*/
	if( dirRel == Dir.GO_FORWARD){
	    if( dirStart == Dir.UP ){ dirEnd = Dir.DOWN; ligth = Lights.LIGHT_WL; }
	    else if( dirStart == Dir.DOWN ){ dirEnd = Dir.UP; ligth = Lights.LIGHT_WR; }
	    else if( dirStart == Dir.LEFT ){ dirEnd = Dir.RIGHT; ligth = Lights.LIGHT_WD; }
	    else if( dirStart == Dir.RIGHT ){ dirEnd = Dir.LEFT; ligth = Lights.LIGHT_WU;}
	    else{ dirEnd = Dir.ERROR; ligth = Lights.LIGHT_NONE; }
	}
	else if( dirRel == Dir.GO_LEFT){
	    if( dirStart == Dir.UP ){ dirEnd = Dir.RIGHT; ligth = Lights.LIGHT_WU; }
	    else if( dirStart == Dir.DOWN ){ dirEnd = Dir.LEFT; ligth = Lights.LIGHT_WD; }
	    else if( dirStart == Dir.LEFT ){ dirEnd = Dir.UP; ligth = Lights.LIGHT_WL; }
	    else if( dirStart == Dir.RIGHT ){ dirEnd = Dir.DOWN; ligth = Lights.LIGHT_WR; }
	    else{ dirEnd = Dir.ERROR; ligth = Lights.LIGHT_NONE; }
	}
	else if( dirRel == Dir.GO_RIGHT){
	    ligth = Lights.LIGHT_NONE;
	    if( dirStart == Dir.UP ){ dirEnd = Dir.LEFT; }
	    else if( dirStart == Dir.DOWN ){ dirEnd = Dir.RIGHT; }
	    else if( dirStart == Dir.LEFT ){ dirEnd = Dir.DOWN; }
	    else if( dirStart == Dir.RIGHT ){ dirEnd = Dir.UP; }
	    else{ dirEnd = Dir.ERROR; }
	}
	else{
	    dirEnd = Dir.ERROR;
	    ligth = Lights.LIGHT_NONE;
	}
    }
    
    private void setStartingPos(int dir_start){
	if(dir_start == Dir.UP){
	    x = -Renderer.SIDE_OFFSET;
	    y = -Renderer.START_OFFSET;
	}
	else if(dir_start == Dir.LEFT){
	    x = -Renderer.START_OFFSET;
	    y = Renderer.SIDE_OFFSET;
	}
	else if(dir_start == Dir.DOWN){
	    x = Renderer.SIDE_OFFSET;
	    y = Renderer.START_OFFSET;
	}
	else if(dir_start == Dir.RIGHT){
	    x = Renderer.START_OFFSET;
	    y = -Renderer.SIDE_OFFSET;
	}
    }
    
    // -----------------------------
    //		STEP
    // -----------------------------
    
    public void step(){
	checkBounds();
	switch (state) {
	    case STATE_PRE:
		stepPre();
		break;
	    case STATE_WAIT:
		stepWait();
		break;
	    case STATE_POST:
		stepPost();
		break;
	    default:
		stepPost();
		break;
	}
    }
    
    private void stepPre(){
	float v = speed;
	if( dirStart == Dir.UP )
	    y+=v;
	else if( dirStart == Dir.DOWN )
	    y-=v;
	else if( dirStart == Dir.LEFT )
	    x+=v;
	else
	    x-=v;
	
	// Detect state change
	if(x*x+y*y < 1.45 * Renderer.JUNCSIZE * Renderer.JUNCSIZE){
	    state = STATE_WAIT;
	    if( dirStart == Dir.UP )
		y = -Renderer.SIDE_OFFSET;
	    else if( dirStart == Dir.DOWN )
		y = Renderer.SIDE_OFFSET;
	    else if( dirStart == Dir.LEFT )
		x = -Renderer.SIDE_OFFSET;
	    else
		x = Renderer.SIDE_OFFSET;
	}
    }
    
    private void stepWait(){
	if( ligth == Lights.LIGHT_NONE)
	    state = STATE_POST; 
	
	int lightstate = lights.getState(ligth);
	if(lightstate == Lights.GREEN)
	   state = STATE_POST; 
	else
	    lights.setLaneBool(ligth);
    }
    
    private void stepPost(){
	float v = speed;
	if( dirEnd == Dir.UP )
	    y-=v;
	else if( dirEnd == Dir.DOWN )
	    y+=v;
	else if( dirEnd == Dir.LEFT )
	    x-=v;
	else
	    x+=v;
    }
    
    private void checkBounds(){
	if( dirEnd == Dir.UP && y < -Renderer.START_OFFSET )
	    isDone = true;
	if( dirEnd == Dir.DOWN && y > Renderer.START_OFFSET )
	    isDone = true;    
	if( dirEnd == Dir.LEFT && x < -Renderer.START_OFFSET )
	    isDone = true;
	if( dirEnd == Dir.RIGHT && x > Renderer.START_OFFSET )
	    isDone = true;  
    }
}
